//! # Tuxcord Reverse Proxy Header Authenthication
#![allow(incomplete_features)]
#![feature(
    anonymous_lifetime_in_impl_trait,
    iterator_try_collect,
    inherent_associated_types,
    stmt_expr_attributes,
    read_buf,
    rwlock_downgrade,
    try_blocks
)]
#![allow(clippy::missing_errors_doc)]

use std::{
    fs::File,
    io::{Read, Write},
    net::TcpListener,
    sync::Arc,
};

use anyhow::Context;
use clap::Parser;

pub mod args;
pub mod config;
pub mod constants;
#[cfg(feature = "ipc")]
pub mod ipc;
pub mod listener;
pub mod utils;

use args::Args;

fn main() -> anyhow::Result<()> {
    let args = Args::parse();

    if !args.config.exists() {
        println!(
            "{:?} doesn't exist, creating a default config",
            &args.config
        );
        File::create(args.config)
            .context("failure creating the config file")?
            .write_all(constants::DEFAULT_CONFIG)
            .context("failure writing the contents to the config file")?;
        return Ok(());
    }

    let mut config_file = File::open(&args.config).context("failure opening the config file")?;
    let mut config = String::new();
    config_file
        .read_to_string(&mut config)
        .context("failure reading the config file")?;
    let config: config::Schema = toml::from_str(&config).context("invalid config file")?;

    #[cfg(feature = "ipc")]
    if let Some(args::Commands::Ipc) = args.command {
        ipc::start_client(config)?;
        return Ok(());
    }

    println!("config: {config:#?}");
    let listener = TcpListener::bind(config.listen_at).context("failure tcp listening")?;
    let config_arc = Arc::new(config); // will also serve as a counter

    #[cfg(feature = "ipc")]
    ipc::handle_daemon(config_arc.clone());

    listener::start_listener_loop(&config_arc, &listener);
}
