pub mod headers {
    use std::borrow::{Borrow, Cow};

    pub trait HeadersExt<'a> {
        fn expect_body(&self) -> bool {
            self.has_any(["conten-length", "transfer-encoding"])
        }
        fn has_any(&self, headers: impl IntoIterator<Item = &'_ str>) -> bool {
            headers.into_iter().any(|h| self.has(h))
        }
        fn has_all(&self, headers: impl IntoIterator<Item = &'_ str>) -> bool {
            headers.into_iter().all(|h| self.has(h))
        }
        fn has(&self, header: &str) -> bool;
        fn header(&self, header: &str) -> Option<Cow<'a, str>>;
    }

    impl<'a, T: Borrow<[httparse::Header<'a>]>> HeadersExt<'a> for T {
        fn has(&self, header: &str) -> bool {
            self.borrow()
                .iter()
                .any(|h| h.name.eq_ignore_ascii_case(header))
        }

        fn header(&self, header: &str) -> Option<Cow<'a, str>> {
            self.borrow()
                .iter()
                .find(|h| h.name.eq_ignore_ascii_case(header))
                .map(|h| String::from_utf8_lossy(h.value))
        }
    }
}
